@@ -22,7 +22,7 @@ ly_lines <- function(
   validate_fig(fig, "ly_lines")
 
   ## deal with possible named inputs from a data source
-  bv <- b_eval(data)
+  # bv <- b_eval(data)
   args <- sub_names(fig, data,
     grab(
       x,
@@ -41,7 +41,7 @@ ly_lines <- function(
   )
   args$params$glyph <- "line"
 
-  if(missing(color) && !is.null(args$params$line_color))
+  if (missing(color) && !is.null(args$params$line_color))
     args$params$color <- NULL
 
   # args$params$alpha <- alpha
@@ -64,7 +64,8 @@ ly_lines <- function(
     # split works with a data.frame as the groups.
     split_list <- split(
       seq_along(args$data$x),
-      as.data.frame(group_dt[names(groupable)])
+      as.data.frame(group_dt[names(groupable)]),
+      drop = TRUE
     )
   } else {
     # no groups to split on.  will split on "one" group
@@ -74,7 +75,7 @@ ly_lines <- function(
 
   axis_type_range <- get_glyph_axis_type_range(args$data$x, args$data$y)
   mc <- attr(fig, "ly_call")
-  if(is.null(mc)) {
+  if (is.null(mc)) {
     mc <- lapply(match.call(), deparse)
   }
 
@@ -82,11 +83,11 @@ ly_lines <- function(
     arg_obj <- subset_arg_obj(args, split_list[[ii]])
 
     ## b_eval will repeat these, but the line glyph doesn't like this
-    if(length(unique(arg_obj$params$color)) == 1)
+    if (length(unique(arg_obj$params$color)) == 1)
       arg_obj$params$color <- subset_with_attributes(arg_obj$params$color, 1)
-    if(length(unique(args$params$type)) == 1)
+    if (length(unique(args$params$type)) == 1)
       arg_obj$params$type <- subset_with_attributes(arg_obj$params$type, 1)
-    if(length(unique(args$params$width)) == 1)
+    if (length(unique(args$params$width)) == 1)
       arg_obj$params$width <- subset_with_attributes(arg_obj$params$width, 1)
 
     arg_obj$params <- resolve_line_args(fig, arg_obj$params)
@@ -148,7 +149,7 @@ ly_segments <- function(fig, x0, y0, x1, y1, data = figure_data(fig),
   )
   args$params$glyph <- "segment"
 
-  if(missing(color) && !is.null(args$params$line_color))
+  if (missing(color) && !is.null(args$params$line_color))
     args$params$color <- NULL
 
   args$params <- resolve_line_args(fig, args$params)
@@ -186,6 +187,23 @@ ly_segments <- function(fig, x0, y0, x1, y1, data = figure_data(fig),
 #' @template par-lnamegroup
 #' @template dots-line
 #' @example man-roxygen/ex-lines.R
+#' @examples
+#' # abline with mixed axes for h and v
+#' figure() %>%
+#'   ly_points(1:26, letters) %>%
+#'   ly_abline(h = "j") %>%
+#'   ly_abline(v = 10)
+#'
+#' # multiple hv lines
+#' figure() %>%
+#'   ly_points(1:10) %>%
+#'   ly_abline(v = 1:10) %>%
+#'   ly_abline(h = 1:10)
+#'
+#' # multiple ab lines
+#' figure() %>%
+#'   ly_points(0:10) %>%
+#'   ly_abline(0, seq(0, 1, by = 0.1))
 #' @family layer functions
 #' @export
 ly_abline <- function(
@@ -212,7 +230,7 @@ ly_abline <- function(
   )
   args$params$glyph <- "segment"
 
-  if(missing(color) && !is.null(args$params$line_color)) {
+  if (missing(color) && !is.null(args$params$line_color)) {
     args$params$color <- NULL
   }
 
@@ -223,30 +241,34 @@ ly_abline <- function(
 
   x_axis_type <- "numeric"
   y_axis_type <- "numeric"
+  if (!is.null(h) || !is.null(v)) {
+    x_axis_type <- fig$x$spec$x_axis_type
+    y_axis_type <- fig$x$spec$y_axis_type
+  }
 
   # manage data
-  if(!is.null(coef) || inherits(a, "lm")) {
-    if(is.null(coef))
+  if (!is.null(coef) || inherits(a, "lm")) {
+    if (is.null(coef))
       coef <- a
-    if(inherits(coef, "lm"))
+    if (inherits(coef, "lm"))
       coef <- coef(coef)
     coef <- as.numeric(coef)
     a <- coef[1]
     b <- coef[2]
   }
 
-  if(!is.null(a) && !is.null(b)) {
+  if (!is.null(a) && !is.null(b)) {
     nn <- max(c(length(a), length(b)))
-    if(length(a) < nn)
+    if (length(a) < nn)
       a <- rep(a, nn)[1:nn]
-    if(length(b) < nn)
+    if (length(b) < nn)
       b <- rep(b, nn)[1:nn]
     x0 <- rep(0, nn)
     y0 <- a
     x1 <- rep(1, nn)
     y1 <- b * x1 + a
-  } else if(!is.null(h)) {
-    if(inherits(h, c("Date", "POSIXt"))) {
+  } else if (!is.null(h)) {
+    if (inherits(h, c("Date", "POSIXt"))) {
       y_axis_type <- "datetime"
       h <- to_epoch(h)
     }
@@ -255,8 +277,8 @@ ly_abline <- function(
     y0 <- h
     x1 <- rep(1, nn)
     y1 <- h
-  } else if(!is.null(v)) {
-    if(inherits(v, c("Date", "POSIXt"))) {
+  } else if (!is.null(v)) {
+    if (inherits(v, c("Date", "POSIXt"))) {
       x_axis_type <- "datetime"
       v <- to_epoch(v)
     }
@@ -268,38 +290,51 @@ ly_abline <- function(
   }
 
   defer_fn <- function(data, xlim, ylim) {
-    if(length(data$x0[[1]]) == 1) {
-      if(data$x0 == "x0")
+    if (length(data$x0) == 1) {
+      if (data$x0 == "x0")
         return(data)
-    } else if(length(data$x0[[1]]) == 0) {
+    } else if (length(data$x0) == 0) {
       return(data)
     }
-    # unlist because of json encoding issues
-    # (json wants each as a list but we want to work with scalars here)
-    data <- unlist(data, recursive = FALSE)
-    if(all(data$x0 == data$x1)) {
+    if (is.list(data$x0))
+      data <- unlist(data, recursive = FALSE)
+    if (all(data$x0 == data$x1)) {
       ## vertical lines
-      data$y0 <- rep(ylim[1], length(data$y0))
-      data$y1 <- rep(ylim[2], length(data$y1))
-    } else if(all(data$y0 == data$y1)) {
+      lo <- head(ylim, 1)
+      up <- tail(ylim, 1)
+      if (is.character(lo)) {
+        lo <- paste0(lo, ":0")
+        up <- paste0(up, ":1")
+      }
+      data$y0 <- rep(lo, length(data$y0))
+      data$y1 <- rep(up, length(data$y1))
+    } else if (all(data$y0 == data$y1)) {
       ## horizontal line
-      data$x0 <- rep(xlim[1], length(data$x0))
-      data$x1 <- rep(xlim[2], length(data$x1))
+      lo <- head(xlim, 1)
+      up <- tail(xlim, 1)
+      if (is.character(lo)) {
+        lo <- paste0(lo, ":0")
+        up <- paste0(up, ":1")
+      }
+      data$x0 <- rep(lo, length(data$x0))
+      data$x1 <- rep(up, length(data$x1))
     } else {
       ## line
       b <- (data$y1 - data$y0) / (data$x1 - data$x0)
       a <- data$y1 - b * data$x1
       nn <- length(a)
-      data$x0 <- rep(xlim[1], nn)
-      data$x1 <- rep(xlim[2], nn)
+      data$x0 <- rep(head(xlim, 1), nn)
+      data$x1 <- rep(tail(xlim, 1), nn)
       data$y0 <- data$x0 * b + a
       data$y1 <- data$x1 * b + a
     }
     # now below wrap each result with list so json encoding is happy
-    data$x0 <- list(data$x0)
-    data$x1 <- list(data$x1)
-    data$y0 <- list(data$y0)
-    data$y1 <- list(data$y1)
+    if (length(data$x0) == 1) {
+      data$x0 <- list(data$x0)
+      data$x1 <- list(data$x1)
+      data$y0 <- list(data$y0)
+      data$y1 <- list(data$y1)
+    }
     data
   }
 
@@ -353,9 +388,12 @@ ly_curve <- function(
     expr <- call(as.character(sexpr), as.name(xname))
   } else {
     yname <- deparse(sexpr)
-    if (!((is.call(sexpr) || is.expression(sexpr)) && xname %in%
-      all.vars(sexpr)))
-      stop(gettextf("'expr' must be a function, or a call or an expression containing '%s'", xname), domain = NA)
+    chk1 <- is.call(sexpr) || is.expression(sexpr)
+    chk <- !(chk1 && xname %in% all.vars(sexpr))
+    if (chk)
+      stop(
+        gettextf("'expr' must be a function, or a call or an expression containing '%s'",
+          xname), domain = NA)
     expr <- sexpr
   }
 
@@ -382,7 +420,7 @@ ly_curve <- function(
   ## see if any options won't be used and give a message
   check_opts(args$params, "line", formals = names(formals(ly_curve)))
 
-  if(missing(color) && !is.null(args$params$line_color))
+  if (missing(color) && !is.null(args$params$line_color))
     args$params$color <- NULL
 
   args$params <- resolve_line_args(fig, args$params)
@@ -505,7 +543,7 @@ ly_ray <- function(
     )
   )
 
-  if(missing(color) && !is.null(args$params$line_color)) {
+  if (missing(color) && !is.null(args$params$line_color)) {
     args$params$color <- NULL
   }
 
@@ -573,9 +611,9 @@ ly_bezier <- function(
       dots = lazy_dots(...)
     )
   )
-  args$params$glyph = "bezier"
+  args$params$glyph <- "bezier"
 
-  if(missing(color) && !is.null(args$params$line_color)) {
+  if (missing(color) && !is.null(args$params$line_color)) {
     args$params$color <- NULL
   }
 
@@ -646,7 +684,7 @@ ly_quadratic <- function(
   )
   args$params$glyph <- "quadratic"
 
-  if(missing(color) && !is.null(args$params$line_color)) {
+  if (missing(color) && !is.null(args$params$line_color)) {
     args$color <- NULL
   }
 
@@ -713,14 +751,14 @@ ly_multi_line <- function(
   )
   args$params$glyph <- "line"
 
-  if(missing(color) && !is.null(args$params$line_color)) {
+  if (missing(color) && !is.null(args$params$line_color)) {
     args$color <- NULL
   }
 
   ## see if any options won't be used and give a message
   # can't pass in color, alpha, width, or type
-  good_names = names(args$params)
-  good_names = good_names[! (good_names %in% c("color", "alpha", "width", "type"))]
+  good_names <- names(args$params)
+  good_names <- good_names[! (good_names %in% c("color", "alpha", "width", "type"))]
   check_opts(args$params[good_names], "multi_line")
 
   args$params <- resolve_line_args(fig, args$params)
